==> tldr_tools-0.2.2.4-py3-none-any.whl: tldr_tools/tldr_submit.py <==
# tldr_api.py
import argparse
import logging
from tldr_tools.tldr_endpoint import *
from tldr_tools.tldr_status import check_job_status
from dotenv import load_dotenv

# Set up logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

MODULE_CONFIG = {
    "dockopt": {
        "endpoint": "submit/dockopt",
        "description": "Optimization pipeline for DOCK38",
        "required_files": {
            "recpdb": "recpdb",
            "xtalpdb": "xtalpdb",
            "activestgz": "activestgz",
            "decoystgz": "decoystgz",
        },
        "optional": ["memo"],
        "cli_args": [
            {"name": "--activestgz", "help": "Path to actives.tgz file.", "required": True},
            {"name": "--decoystgz", "help": "Path to decoys.tgz file.", "required": True},
            {"name": "--recpdb", "help": "Path to receptor PDB file.", "required": True},
            {"name": "--xtalpdb", "help": "Path to xtal ligand PDB file.", "required": True},
            {"name": "--memo", "help": "Optional memo text.", "required": False},
        ],
    },
    "build": {
        "endpoint": "submit/build3d38",
        "description": "Prepare a 3D library for docking in up to four formats used by popular docking programs using DOCK3.8 pipeline.",
        "required_files": {
            "input": "input.txt",
        },
        "optional": ["memo"],
        "cli_args": [
            {"name": "--input", "help": "File of SMILES ([SMILES] [COMPOUND_NAME] per line).", "required": True},
            {"name": "--memo", "help": "Optional memo text.", "required": False},
        ],
    },
    "decoys": {
        "endpoint": "submit/dudez",
        "description": "Decoy generation module for active compound generation.",
        "required_files": {
            "activesism": "actives.ism",
            "decoygenin": "decoy_generation",
        },
        "optional": ["memo"],
        "cli_args": [
            {"name": "--activesism", "help": "Path to active.ism file.", "required": True},
            {"name": "--decoygenin", "help": "Path to decoy_generation.in file.", "required": True},
            {"name": "--memo", "help": "Optional memo text.", "required": False},
        ],
    },
}


def download_decoys_if_completed(api_manager: APIManager, job_number: str, output_dir: str):
    """Checks job status and downloads decoys if the job is completed."""
    try:
        if api_manager.status_by_job_no(job_number) == "Completed":
            api_manager.download_decoys(job_number, output_path=output_dir)
        else:
            logger.error(f"Job {job_number} is not completed, cannot download decoys.")
    except Exception as e:
        logger.error(f"Error checking job status or downloading decoys: {e}")

def add_module_arguments(parser, module_name, module_config):
    """Adds CLI arguments for a module to an argparse parser."""
    cli_args = module_config[module_name]["cli_args"]
    for arg in cli_args:
        parser.add_argument(arg["name"], help=arg["help"], required=arg["required"])


from typing import Optional
import logging

logger = logging.getLogger(__name__)

def submit_module(api_manager: APIManager, module: str, **kwargs) -> Optional[str]:
    """
    Submits a module with the provided arguments and returns the job number.
    :param api_manager: Instance of APIManager to handle submission.
    :param module: Name of the module to submit.
    :param kwargs: Module-specific arguments.
    :return: Submitted job number, or None on failure.
    """
    def path_to_payload(path: str):
        """Helper to open a file in binary mode for uploading."""
        try:
            return open(path, "rb")
        except FileNotFoundError:
            logger.error(f"File not found: {path}")
            return None

    # Load module-specific configuration
    if module not in MODULE_CONFIG:
        logger.error(f"Unknown module: {module}")
        return None

    config = MODULE_CONFIG[module]
    files = {}

    try:
        # Prepare required files
        for file_arg, payload_name in config.get("required_files", {}).items():
            if file_arg not in kwargs or not kwargs[file_arg]:
                logger.error(f"Missing required file: {file_arg}")
                return None

            file_obj = path_to_payload(kwargs[file_arg])
            if not file_obj:
                return None  # Error logged by `path_to_payload`

            files[payload_name] = (None, file_obj)

        # Add optional parameters
        for optional_arg in config.get("optional", []):
            if optional_arg in kwargs and kwargs[optional_arg]:
                files[optional_arg] = (None, kwargs[optional_arg])

        # Submit the module
        response = api_manager.post_request(
            TLDREndpoints.get_endpoint(config["endpoint"]), files=files
        )

        return response

    except Exception as e:
        logger.error(f"Error during submission: {e}")
        return None

    finally:
        # Ensure all file handles are closed
        for key, value in files.items():
            if isinstance(value, tuple) and hasattr(value[1], "close"):
                value[1].close()


def add_module_arguments(parser, module_name, module_config):
    """Dynamically add arguments based on module config."""
    module = module_config.get(module_name, {})
    if not module:
        raise ValueError(f"Module {module_name} not found in the configuration.")

    # Add arguments specified in cli_args
    for arg in module.get("cli_args", []):
        parser.add_argument(
            arg["name"], 
            type=str, 
            required=arg.get("required", False), 
            help=arg.get("help", "No description provided.")
        )

def main():
    parser = argparse.ArgumentParser(description="Submit and manage docking tasks via TLDR API.")
    parser.add_argument("--list-modules", action="store_true", help="List all available modules and exit.")
    parser.add_argument("--module", choices=MODULE_CONFIG.keys(), help="Module type to submit.")
    parser.add_argument("--job-number", help="Job number to check status.")
    parser.add_argument("--output-dir", default="decoys", help="Directory to store downloaded decoys.")
    
    args, unknown_args = parser.parse_known_args() 

    api_manager = APIManager()  # Initialize API manager

    # Handle listing modules
    if args.list_modules:
        print("Available modules:")
        for module_name, config in MODULE_CONFIG.items():
            print(f"- {module_name}: {config['description']}")
            for arg in config.get("cli_args", []):
                print(f"   {arg['name']} - {arg.get('help', 'No description provided.')}")
        return 

    # # Handle job status check
    # if args.job_number:
    #     check_job_status(api_manager, args.job_number)
    #     return

    # Add dynamic arguments for the selected module
    module_name = args.module
    if not module_name:
        parser.error("You must specify a module using --module.")
    
    module_parser = argparse.ArgumentParser()
    add_module_arguments(module_parser, module_name, MODULE_CONFIG)
    
    # Parse remaining arguments for the module
    module_args = module_parser.parse_args(unknown_args)

    # Submit the module
    response = submit_module(api_manager, module_name, **vars(module_args))  # Unpack the args dictionary

    if response.text:
        logger.info("Job submitted, but unsure if it went through (this is expected). Checking if identified job is running...")
        submitted_job = api_manager.url_to_job_no(response.url)
        job_status = check_job_status(api_manager, submitted_job)

        print(job_status)
        
        if job_status in ['Submitted', 'Running']:
            logger.info(f"Job {submitted_job} is {job_status} and submitted successfully!")
        else:
            logger.error(f"Job {submitted_job} status is unrecognized: {job_status}.")
    else:
        logger.error("Job failed to submit.")


if __name__ == "__main__":
    main()

==> tldr_tools-0.2.4.2-py3-none-any.whl: tldr_tools/tldr_submit.py <==
import argparse
import logging
import os
import csv
from typing import Optional
from dotenv import load_dotenv
from tldr_tools.tldr_endpoint import APIManager, TLDREndpoints
from tldr_tools.tldr_status import check_job_status

# Set up logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

MODULE_CONFIG = {
    "dockopt": {
        "endpoint": "submit/dockopt",
        "description": "Optimization pipeline for DOCK38",
        "required_files": {
            "recpdb": "recpdb",
            "xtalpdb": "xtalpdb",
            "activestgz": "activestgz",
            "decoystgz": "decoystgz",
        },
        "optional": ["memo"],
        "cli_args": [
            {"name": "--activestgz", "help": "Path to actives.tgz file.", "required": True},
            {"name": "--decoystgz", "help": "Path to decoys.tgz file.", "required": True},
            {"name": "--recpdb", "help": "Path to receptor PDB file.", "required": True},
            {"name": "--xtalpdb", "help": "Path to xtal ligand PDB file.", "required": True},
            {"name": "--memo", "help": "Optional memo text.", "required": False},
        ],
    },
    "build": {
        "endpoint": "submit/build3d38",
        "description": "Prepare a 3D library for docking in up to four formats used by popular docking programs using DOCK3.8 pipeline.",
        "required_files": {
            "input": "input.txt",
        },
        "optional": ["memo"],
        "cli_args": [
            {"name": "--input", "help": "File of SMILES ([SMILES] [COMPOUND_NAME] per line).", "required": True},
            {"name": "--memo", "help": "Optional memo text.", "required": False},
        ],
    },
    "decoys": {
        "endpoint": "submit/dudez",
        "description": "Decoy generation module for active compound generation.",
        "required_files": {
            "activesism": "actives.ism",
            "decoygenin": "decoy_generation.in",
        },
        "optional": ["memo"],
        "cli_args": [
            {"name": "--activesism", "help": "Path to actives.ism file.", "required": True},
            {"name": "--decoygenin", "help": "Path to decoy_generation.in file.", "required": True},
            {"name": "--memo", "help": "Optional memo text.", "required": False},
        ],
    },
}

def export_job_to_csv(csv_path: str, memo: str, job_type: str, job_no: str):
    """
    Append a job record to a CSV file with columns: memo, job_type, job_no.
    If the file exists, checks column compatibility before appending.
    """
    expected_fields = ["memo", "job_type", "job_no"]
    file_exists = os.path.isfile(csv_path)

    # If file exists, validate header
    if file_exists:
        with open(csv_path, newline='') as csvfile:
            reader = csv.reader(csvfile)
            header = next(reader, None)
        if header != expected_fields:
            logger.error(f"CSV file {csv_path} has incompatible columns: {header}. Expected {expected_fields}.")
            return

    # Open for appending (create with header if new)
    with open(csv_path, 'a', newline='') as csvfile:
        writer = csv.DictWriter(csvfile, fieldnames=expected_fields)
        if not file_exists:
            writer.writeheader()
        writer.writerow({
            "memo": memo,
            "job_type": job_type,
            "job_no": job_no,
        })
    logger.info(f"Exported job {job_no} to CSV {csv_path}.")


def add_module_arguments(parser, module_name, module_config):
    """Dynamically add arguments based on module config."""
    module = module_config.get(module_name, {})
    if not module:
        raise ValueError(f"Module {module_name} not found in the configuration.")
    for arg in module.get("cli_args", []):
        parser.add_argument(
            arg["name"],
            type=str,
            required=arg.get("required", False),
            help=arg.get("help", "No description provided."),
        )


def main():
    parser = argparse.ArgumentParser(description="Submit and manage docking tasks via TLDR API.")
    parser.add_argument("--list-modules", action="store_true", help="List all available modules and exit.")
    parser.add_argument("--module", choices=MODULE_CONFIG.keys(), help="Module type to submit.")
    parser.add_argument("--export-csv", help="Path to CSV file to append job records.")

    args, unknown_args = parser.parse_known_args()

    # Handle listing modules
    if args.list_modules:
        print("Available modules:")
        for m, config in MODULE_CONFIG.items():
            print(f"- {m}: {config['description']}")
            for arg in config.get("cli_args", []):
                print(f"   {arg['name']} - {arg.get('help', 'No description provided.')}")
        return

    # Ensure module selection
    if not args.module:
        parser.error("You must specify a module using --module.")
    module_name = args.module

    # Build module-specific parser
    module_parser = argparse.ArgumentParser(add_help=False)
    add_module_arguments(module_parser, module_name, MODULE_CONFIG)
    module_args = module_parser.parse_args(unknown_args)

    # Initialize API manager
    api_manager = APIManager()

    # Submit module
    response = submit_module(api_manager, module_name, **vars(module_args))
    if not response:
        logger.error("Job failed to submit.")
        return

    # Check submission and status
    if hasattr(response, 'text') and response.text:
        logger.info("Job submitted, but unsure if it went through (this is expected). Checking if identified job is running...")
        submitted_job = api_manager.url_to_job_no(response.url)
        job_status = check_job_status(api_manager, submitted_job)

        print(job_status)
        if job_status in ['Submitted', 'Running']:
            logger.info(f"Job {submitted_job} is {job_status} and submitted successfully!")
            # Export to CSV if requested
            if args.export_csv:
                memo_value = getattr(module_args, 'memo', '') or ''
                export_job_to_csv(args.export_csv, memo_value, module_name, submitted_job)
        else:
            logger.error(f"Job {submitted_job} status is unrecognized: {job_status}.")
    else:
        logger.error("Job failed to submit.")


if __name__ == "__main__":
    main()
